#!/usr/bin/env python

'''
This module provides basic utilities for the modules server_classes.py and start_server.py.

Reading University
MSc in Network Centered Computing
a.weise - a.weise@reading.ac.uk - December 2005
'''

import ConfigParser, string
import time, os

def LoadConfig(file_name, config={}):
    """
    returns a dictionary with key's of the form
    <section>.<option> and the values
    
    source: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65334
    """
    config = config.copy()
    cp = ConfigParser.ConfigParser()
    cp.read(file_name)
    for sec in cp.sections():
        name = string.lower(sec)
        for opt in cp.options(sec):
            config[name + "." + string.lower(opt)] = string.strip(cp.get(sec, opt))
    return config

def check_ip(ip):
    '''
    This function checks if a given IP is valid.
    '''
    try:
        ip = ip.split(".")
    except AttributeError:
        return -1
    
    for i in range(len(ip)):
        check =  ip[i].find("0", 0, 1)
        if -1 != check and 1 < len(ip[i]):
            return -1
        try:
            ip[i] = int(ip[i])
        except ValueError:
            return -1
        if ip[i] >= 0 and ip[i] <= 255:
            pass
        else:
            return -1
        
    return 0

def get_keywords(filus):
    '''
    This function extracts keywords from a give file!
    '''
    keys = []
    
    try:
        file_fd = file(filus, 'r')
    except IOError, e:
        print "Problem with keyword file -> ",  e
        return -1
    
    content = file_fd.readlines()# save file content as list (1 line == 1 entry)

    file_fd.close()

    content = remove_item(content, "#")
    content = remove_item(content, "\n")

    for i in range(len(content)):
        content[i] = content[i].strip()
        content[i] = content[i].rstrip(",")
        content[i] = content[i].split(",")
        for a in range(len(content[i])):
            keys.append(content[i][a])
    
    for i in range(len(keys)):
        keys[i] = keys[i].strip() # remove whitespace
        keys[i] = keys[i].split(":")
        
    return keys
        
def remove_item(listus, item):
    '''
    This function removes an item for a list (2 dimentional) as a rekursive function.
    '''
    
    while(1):
        
        for i in range(len(listus)):
            if -1 != listus[i].find(item, 0, 1):
                del listus[i]
                remove_item(listus, item)
                break
        else:
            break
        
    return listus

def list_to_string(listus):
    '''
    This function converts the keyword list (2 dimensional array) to a keyword string (keywords comma separated), so the string is writable into the keyword file.
    '''
    str_listus = ''
    
    for i in range(len(listus)):
        if 1 == len(listus[i]):
            str_listus += listus[i][0]+", "
        elif 2 == len(listus[i]):
            str_listus += listus[i][0]+":"+listus[i][1]+", "
            
    str_listus = str_listus.strip()
    str_listus = str_listus.strip(",")
    
    return str_listus

def delete_file(file_name, verbose):
    '''
    This function deletes a file.
    '''
    try:
        os.remove(file_name)
        return 0
    except:
        if verbose == 1:
            print "%s -> could not delete -> \"%s\"" % (time.ctime(), file_name)
        return -1
    
def usage_exit(progname, msg=None):
    '''
    This function diplays the usage of this program and terminates the program!
    '''
    if msg:
        print msg
        print
    print "usage: python %s [ -h|--help -c|--config -v|--verbose -d|--daemon] \n\n" % progname
    os._exit(-1)
    
def find(search, listus):
    '''
    This function finds an item within a list (1 dimensional).
    '''
    for i in range(len(listus)):
        if listus[i] == search:
            return listus[i]
    return None 